#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
File containing definitions for the casa_matrix routes of the Workspaces REST API.
"""

import http
from json import JSONDecodeError

from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config


def parse_body(request: Request) -> dict:
    try:
        body = request.json_body
    except JSONDecodeError:
        # Caller will handle empty/optional parameters
        body = {}
    return body


@view_config(route_name="get_casa_version", renderer="json")
def get_casa_version(request: Request) -> Response:
    """
    Pyramid view that gets a CASA version for processing.

    :param request: GET request
    :return: Response containing the CASA version and path
        or a 404 response (HTTPNotFound) if one isn't returned
    """
    body = parse_body(request)

    if "capability" not in body:
        return HTTPBadRequest(detail=f"Capability not given")

    params = {
        "version": body["version"] if "version" in body else None,
        "capability": body["capability"],
        "telescope": body["telescope"] if "telescope" in body else None,
    }
    params = {k: v for k, v in params.items() if v is not None}

    casa_version = request.casa_matrix_service.get_version(**params)

    if casa_version:
        return Response(status_int=http.HTTPStatus.OK, json_body=casa_version)
    else:
        return HTTPNotFound(detail=f"No CASA version found.")


@view_config(route_name="get_casa_versions", renderer="json")
def get_casa_versions(request: Request) -> Response:
    """
    Pyramid view that gets a list of CASA versions for processing.

    :param request: GET request
    :return: Response containing a list of CASA versions and their paths
        or a 404 response (HTTPNotFound) if none are returned
    """
    body = parse_body(request)

    if "capabilities" not in body:
        return HTTPBadRequest(detail=f"Capabilities not given")

    casa_versions = {}

    for capability in body["capabilities"]:
        params = {
            "version": body["version"] if "version" in body else None,
            "capability": capability,
            "telescope": body["telescope"] if "telescope" in body else None,
        }
        params = {k: v for k, v in params.items() if v is not None}

        casa_versions[capability] = request.casa_matrix_service.get_versions(**params)

    if casa_versions:
        return Response(status_int=http.HTTPStatus.OK, json_body=casa_versions)
    else:
        return HTTPNotFound(detail=f"No CASA versions found.")


@view_config(route_name="get_casa_recipe", renderer="json")
def get_casa_recipe(request: Request) -> Response:
    """
    Pyramid view that gets a CASA recipe for the given capability.

    :param request: GET request
    :return: Response containing the CASA recipe
        or a 404 response (HTTPNotFound) if one isn't returned
    """
    body = parse_body(request)

    if "capability" not in body:
        return HTTPBadRequest(detail=f"Capability not given")

    params = {
        "version": body["version"] if "version" in body else None,
        "capability": body["capability"],
        "telescope": body["telescope"] if "telescope" in body else None,
        "version_path": body["version_path"] if "version_path" in body else None,
    }
    params = {k: v for k, v in params.items() if v is not None}

    casa_recipe = request.casa_matrix_service.get_recipe(**params)

    if casa_recipe:
        return Response(status_int=http.HTTPStatus.OK, json_body=casa_recipe)
    else:
        return HTTPNotFound(detail=f"No CASA recipe found.")


@view_config(route_name="add_casa_version", renderer="json")
def add_casa_version(request: Request) -> Response:
    """
    Pyramid view that adds a CASA version to the matrix.

    :param request: POST request
    :return: Response containing true if the new matrix version was added
        or a 404 response (HTTPNotFound) if one isn't added
    """
    body = parse_body(request)

    if "version" not in body:
        return HTTPBadRequest(detail=f"CASA version not given")

    if "capabilities" in body and not isinstance(body["capabilities"], list):
        return HTTPBadRequest(detail=f"Capabilities given aren't a valid list")

    params = {
        "version": body["version"],
        "capabilities": body["capabilities"] if "capabilities" in body else [],
        "is_cluster_compatible": body["is_cluster_compatible"] if "is_cluster_compatible" in body else True,
    }
    added = request.casa_matrix_service.add_version(**params)

    if added:
        return Response(status_int=http.HTTPStatus.OK, json_body=added)
    else:
        raise HTTPNotFound(detail=f"Matrix version not added")


@view_config(route_name="update_casa_version", renderer="json")
def update_casa_version(request: Request) -> Response:
    """
    Pyramid view that updates a CASA version in the matrix.

    :param request: POST request
    :return: Response containing true if the matrix version was updated
        or a 404 response (HTTPNotFound) if one isn't updated
    """
    body = parse_body(request)

    if "version" not in body:
        return HTTPBadRequest(detail=f"CASA version not given")

    if "capabilities" in body and not isinstance(body["capabilities"], list):
        return HTTPBadRequest(detail=f"Capabilities given aren't a valid list")

    params = {
        "version": body["version"],
        "capabilities": body["capabilities"] if "capabilities" in body else [],
        "is_cluster_compatible": body["is_cluster_compatible"] if "is_cluster_compatible" in body else True,
    }
    updated = request.casa_matrix_service.update_version(**params)

    if updated:
        return Response(status_int=http.HTTPStatus.OK, json_body=updated)
    else:
        raise HTTPNotFound(detail=f"Matrix version not updated")


@view_config(route_name="delete_casa_version", renderer="json")
def delete_casa_version(request: Request) -> Response:
    """
    Pyramid view that deletes a CASA version from the matrix.

    :param request: POST request
    :return: Response containing true if version was deleted from the matrix
        or a 404 response (HTTPNotFound) if one isn't deleted
    """
    body = parse_body(request)

    if "version" not in body:
        return HTTPBadRequest(detail=f"CASA version not given")

    deleted = request.casa_matrix_service.delete_version(body["version"])

    if deleted:
        return Response(status_int=http.HTTPStatus.OK, json_body=deleted)
    else:
        return HTTPNotFound(detail=f"Matrix version not deleted")


@view_config(route_name="make_casa_links", renderer="json")
def make_casa_links(request: Request) -> Response:
    """
    Pyramid view that refreshes the list of symlinks pointing to installed CASA versions.

    :param request: POST request
    :return: Response containing the list of installed versions that were linked
        or a 404 response (HTTPNotFound) if no links were updated
    """
    found_versions = request.casa_matrix_service.make_links()

    if found_versions:
        return Response(status_int=http.HTTPStatus.OK, json_body=found_versions)
    else:
        return HTTPNotFound(detail=f"CASA links not updated")
